import collections
import datetime
import pydicom
import pydicom.dataelem
import random


class UIAnonymizer:
    def __init__(self):
        """\
        Create a new UIAnonymizer.
        """
        self._ui_map = collections.defaultdict(self._new_ui)

    def __call__(self, dataset, data_element):
        """\
        Potentially anonymize a single DataElement, replacing its
        value with something that obscures the patient's identity.

        Parameters
        ----------
        dataset : pydicom.dataset.Dataset
            The dataset to operate on.

        data_element : pydicom.dataset.DataElement
            The current element. Will be anonymized if its VR is UI.
            If multi-valued, each item will be anonymized
            independently.

        Returns
        -------
        True if the element was anonymized, or False if not.
        """
        if (
            data_element.VR != "UI"
            or not data_element.value
            or pydicom.datadict.keyword_for_tag(data_element.tag).endswith("ClassUID")
            or data_element.tag == pydicom.datadict.tag_for_keyword("TransferSyntaxUID")
        ):
            return False

        if isinstance(data_element.value, pydicom.multival.MultiValue):
            data_element.value = list([self._ui_map[v] for v in data_element.value])
        else:
            data_element.value = self._ui_map[data_element.value]
        return True

    def _new_ui(self):
        date_part = datetime.datetime.utcnow().strftime("%Y%m%d%H%M%S")
        return "2." + date_part + "." + str(random.randint(1e45, 1e46 - 1))



import pydicom


class IDAnonymizer:
    _alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"

    def __init__(self, randomizer, id_prefix, id_suffix, *keywords):
        """\
        Create a new IDAnonymizer.

        Parameters
        ----------
        randomizer : dicognito.randomizer.Randomizer
            Provides a source of randomness.
        id_prefix : str
            A prefix to add to all unstructured ID fields, such as Patient
            ID, Accession Number, etc.
        id_suffix : str
            A prefix to add to all unstructured ID fields, such as Patient
            ID, Accession Number, etc.
        keywords : list of str
            All of the keywords for elements to be anonymized. Only
            elements with matching keywords will be updated.
        """
        self.randomizer = randomizer
        self.id_prefix = id_prefix
        self.id_suffix = id_suffix
        self.issuer_tag = pydicom.datadict.tag_for_keyword("IssuerOfPatientID")
        self.id_tags = [pydicom.datadict.tag_for_keyword(tag_name) for tag_name in keywords]

        total_affixes_length = len(self.id_prefix) + len(self.id_suffix)
        self._indices_for_randomizer = [len(self._alphabet)] * (12 - total_affixes_length)

    def __call__(self, dataset, data_element):
        """\
        Potentially anonymize a single DataElement, replacing its
        value with something that obscures the patient's identity.

        Parameters
        ----------
        dataset : pydicom.dataset.Dataset
            The dataset to operate on.

        data_element : pydicom.dataset.DataElement
            The current element. Will be anonymized if it has a value
            and if its keyword matches one of the keywords supplied when
            creating this anonymizer or matches IssuerOfPatientID.

            The element may be multi-valued, in which case each item is
            anonymized independently.

        Returns
        -------
        True if the element was anonymized, or False if not.
        """
        if data_element.tag in self.id_tags:
            self._replace_id(data_element)
            return True

        if self._anonymize_mitra_global_patient_id(dataset, data_element):
            return True

        if data_element.tag == self.issuer_tag and data_element.value:
            data_element.value = "DICOGNITO"
            return True
        return False

    def _anonymize_mitra_global_patient_id(self, dataset, data_element):
        if data_element.tag.group == 0x0031 and data_element.tag.element % 0x0020 == 0:
            private_tag_group = data_element.tag.element >> 8
            if dataset[(0x0031 << 16) + private_tag_group].value == "MITRA LINKED ATTRIBUTES 1.0":
                self._replace_id(data_element)
                data_element.value = data_element.value.encode()
                return True
        return False

    def _replace_id(self, data_element):
        if isinstance(data_element.value, pydicom.multival.MultiValue):
            data_element.value = [self._new_id(id) for id in data_element.value]
        else:
            data_element.value = self._new_id(data_element.value)

    def _new_id(self, original_value):
        indexes = self.randomizer.get_ints_from_ranges(original_value, *self._indices_for_randomizer)
        id_root = "".join([self._alphabet[i] for i in indexes])
        return self.id_prefix + id_root + self.id_suffix



"""\
dicognito - anonymize DICOM datasets

Invokable as a script, or use the dicognito.anonymizer.Anonymizer class
for finer-grained control around loading and saving DICOM files.
"""

from ._version import __version__

__all__ = ["__version__"]



import pydicom


class PNAnonymizer:
    def __init__(self, randomizer):
        """\
        Create a new PNAnonymizer.

        Parameters
        ----------
        randomizer : dicognito.randomizer.Randomizer
            Provides a source of randomness.
        """
        self.randomizer = randomizer

    def __call__(self, dataset, data_element):
        """\
        Potentially anonymize a single DataElement, replacing its
        value with something that obscures the patient's identity.

        Parameters
        ----------
        dataset : pydicom.dataset.Dataset
            The dataset to operate on.

        data_element : pydicom.dataset.DataElement
            The current element. Will be anonymized if its VR is PN.

        Returns
        -------
        True if the element was anonymized, or False if not.
        """
        if data_element.VR != "PN":
            return False
        if not data_element.value:
            return True

        patient_sex = dataset.get("PatientSex")
        if isinstance(data_element.value, pydicom.multival.MultiValue):
            data_element.value = [self._new_pn(patient_sex, original_name) for original_name in data_element.value]
        else:
            data_element.value = self._new_pn(patient_sex, data_element.value)
        return True

    def _new_pn(self, sex, original_value):
        if sex == "F":
            first_names = self._female_first_names
        elif sex == "M":
            first_names = self._male_first_names
        else:
            first_names = self._all_first_names

        indices = self.randomizer.get_ints_from_ranges(
            original_value, len(self._last_names), len(first_names), len(self._all_first_names)
        )

        return self._last_names[indices[0]] + "^" + first_names[indices[1]] + "^" + self._all_first_names[indices[2]]

    _female_first_names = [
        "MARY",
        "PATRICIA",
        "LINDA",
        "BARBARA",
        "ELIZABETH",
        "JENNIFER",
        "MARIA",
        "SUSAN",
        "MARGARET",
        "DOROTHY",
        "LISA",
        "NANCY",
        "KAREN",
        "BETTY",
        "HELEN",
        "SANDRA",
        "DONNA",
        "CAROL",
        "RUTH",
        "SHARON",
        "MICHELLE",
        "LAURA",
        "SARAH",
        "KIMBERLY",
        "DEBORAH",
        "JESSICA",
        "SHIRLEY",
        "CYNTHIA",
        "ANGELA",
        "MELISSA",
        "BRENDA",
        "AMY",
        "ANNA",
        "REBECCA",
        "VIRGINIA",
        "KATHLEEN",
        "PAMELA",
        "MARTHA",
        "DEBRA",
        "AMANDA",
        "STEPHANIE",
        "CAROLYN",
        "CHRISTINE",
        "MARIE",
        "JANET",
        "CATHERINE",
        "FRANCES",
        "ANN",
        "JOYCE",
        "DIANE",
        "ALICE",
        "JULIE",
        "HEATHER",
        "TERESA",
        "DORIS",
        "GLORIA",
        "EVELYN",
        "JEAN",
        "CHERYL",
        "MILDRED",
        "KATHERINE",
        "JOAN",
        "ASHLEY",
        "JUDITH",
        "ROSE",
        "JANICE",
        "KELLY",
        "NICOLE",
        "JUDY",
        "CHRISTINA",
        "KATHY",
        "THERESA",
        "BEVERLY",
        "DENISE",
        "TAMMY",
        "IRENE",
        "JANE",
        "LORI",
        "RACHEL",
        "MARILYN",
        "ANDREA",
        "KATHRYN",
        "LOUISE",
        "SARA",
        "ANNE",
        "JACQUELINE",
        "WANDA",
        "BONNIE",
        "JULIA",
        "RUBY",
        "LOIS",
        "TINA",
        "PHYLLIS",
        "NORMA",
        "PAULA",
        "DIANA",
        "ANNIE",
        "LILLIAN",
        "EMILY",
        "ROBIN",
        "PEGGY",
        "CRYSTAL",
        "GLADYS",
        "RITA",
        "DAWN",
        "CONNIE",
        "FLORENCE",
        "TRACY",
        "EDNA",
        "TIFFANY",
        "CARMEN",
        "ROSA",
        "CINDY",
        "GRACE",
        "WENDY",
        "VICTORIA",
        "EDITH",
        "KIM",
        "SHERRY",
        "SYLVIA",
        "JOSEPHINE",
        "THELMA",
        "SHANNON",
        "SHEILA",
        "ETHEL",
        "ELLEN",
        "ELAINE",
        "MARJORIE",
        "CARRIE",
        "CHARLOTTE",
        "MONICA",
        "ESTHER",
        "PAULINE",
        "EMMA",
        "JUANITA",
        "ANITA",
        "RHONDA",
        "HAZEL",
        "AMBER",
        "EVA",
        "DEBBIE",
        "APRIL",
        "LESLIE",
        "CLARA",
        "LUCILLE",
        "JAMIE",
        "JOANNE",
        "ELEANOR",
        "VALERIE",
        "DANIELLE",
        "MEGAN",
        "ALICIA",
        "SUZANNE",
        "MICHELE",
        "GAIL",
        "BERTHA",
        "DARLENE",
        "VERONICA",
        "JILL",
        "ERIN",
        "GERALDINE",
        "LAUREN",
        "CATHY",
        "JOANN",
        "LORRAINE",
        "LYNN",
        "SALLY",
        "REGINA",
        "ERICA",
        "BEATRICE",
        "DOLORES",
        "BERNICE",
        "AUDREY",
        "YVONNE",
        "ANNETTE",
        "JUNE",
        "SAMANTHA",
        "MARION",
        "DANA",
        "STACY",
        "ANA",
        "RENEE",
        "IDA",
        "VIVIAN",
        "ROBERTA",
        "HOLLY",
        "BRITTANY",
        "MELANIE",
        "LORETTA",
        "YOLANDA",
        "JEANETTE",
        "LAURIE",
        "KATIE",
        "KRISTEN",
        "VANESSA",
        "ALMA",
        "SUE",
        "ELSIE",
        "BETH",
        "JEANNE",
        "VICKI",
        "CARLA",
        "TARA",
        "ROSEMARY",
        "EILEEN",
        "TERRI",
        "GERTRUDE",
        "LUCY",
        "TONYA",
        "ELLA",
        "STACEY",
        "WILMA",
        "GINA",
        "KRISTIN",
        "JESSIE",
        "NATALIE",
        "AGNES",
        "VERA",
        "WILLIE",
        "CHARLENE",
        "BESSIE",
        "DELORES",
        "MELINDA",
        "PEARL",
        "ARLENE",
        "MAUREEN",
        "COLLEEN",
        "ALLISON",
        "TAMARA",
        "JOY",
        "GEORGIA",
        "CONSTANCE",
        "LILLIE",
        "CLAUDIA",
        "JACKIE",
        "MARCIA",
        "TANYA",
        "NELLIE",
        "MINNIE",
        "MARLENE",
        "HEIDI",
        "GLENDA",
        "LYDIA",
        "VIOLA",
        "COURTNEY",
        "MARIAN",
        "STELLA",
        "CAROLINE",
        "DORA",
        "JO",
        "VICKIE",
        "MATTIE",
        "TERRY",
        "MAXINE",
        "IRMA",
        "MABEL",
        "MARSHA",
        "MYRTLE",
        "LENA",
        "CHRISTY",
        "DEANNA",
        "PATSY",
        "HILDA",
        "GWENDOLYN",
        "JENNIE",
        "NORA",
        "MARGIE",
        "NINA",
        "CASSANDRA",
        "LEAH",
        "PENNY",
        "KAY",
        "PRISCILLA",
        "NAOMI",
        "CAROLE",
        "BRANDY",
        "OLGA",
        "BILLIE",
        "DIANNE",
        "TRACEY",
        "LEONA",
        "JENNY",
        "FELICIA",
        "SONIA",
        "MIRIAM",
        "VELMA",
        "BECKY",
        "BOBBIE",
        "VIOLET",
        "KRISTINA",
        "TONI",
        "MISTY",
        "MAE",
        "SHELLY",
        "DAISY",
        "RAMONA",
        "SHERRI",
        "ERIKA",
        "KATRINA",
        "CLAIRE",
        "LINDSEY",
        "LINDSAY",
        "GENEVA",
        "GUADALUPE",
        "BELINDA",
        "MARGARITA",
        "SHERYL",
        "CORA",
        "FAYE",
        "ADA",
        "NATASHA",
        "SABRINA",
        "ISABEL",
        "MARGUERITE",
        "HATTIE",
        "HARRIET",
        "MOLLY",
        "CECILIA",
        "KRISTI",
        "BRANDI",
        "BLANCHE",
        "SANDY",
        "ROSIE",
        "JOANNA",
        "IRIS",
        "EUNICE",
        "ANGIE",
        "INEZ",
        "LYNDA",
        "MADELINE",
        "AMELIA",
        "ALBERTA",
        "GENEVIEVE",
        "MONIQUE",
        "JODI",
        "JANIE",
        "MAGGIE",
        "KAYLA",
        "SONYA",
        "JAN",
        "LEE",
        "KRISTINE",
        "CANDACE",
        "FANNIE",
        "MARYANN",
        "OPAL",
        "ALISON",
        "YVETTE",
        "MELODY",
        "LUZ",
        "SUSIE",
        "OLIVIA",
        "FLORA",
        "SHELLEY",
        "KRISTY",
        "MAMIE",
        "LULA",
        "LOLA",
        "VERNA",
        "BEULAH",
        "ANTOINETTE",
        "CANDICE",
        "JUANA",
        "JEANNETTE",
        "PAM",
        "KELLI",
        "HANNAH",
        "WHITNEY",
        "BRIDGET",
        "KARLA",
        "CELIA",
        "LATOYA",
        "PATTY",
        "SHELIA",
        "GAYLE",
        "DELLA",
        "VICKY",
        "LYNNE",
        "SHERI",
        "MARIANNE",
        "KARA",
        "JACQUELYN",
        "ERMA",
        "BLANCA",
        "MYRA",
        "LETICIA",
        "PAT",
        "KRISTA",
        "ROXANNE",
        "ANGELICA",
        "JOHNNIE",
        "ROBYN",
        "FRANCIS",
        "ADRIENNE",
        "ROSALIE",
        "ALEXANDRA",
        "BROOKE",
        "BETHANY",
        "SADIE",
        "BERNADETTE",
        "TRACI",
        "JODY",
        "KENDRA",
        "JASMINE",
        "NICHOLE",
        "RACHAEL",
        "CHELSEA",
        "MABLE",
        "ERNESTINE",
        "MURIEL",
        "MARCELLA",
        "ELENA",
        "KRYSTAL",
        "ANGELINA",
        "NADINE",
        "KARI",
        "ESTELLE",
        "DIANNA",
        "PAULETTE",
        "LORA",
        "MONA",
        "DOREEN",
        "ROSEMARIE",
        "ANGEL",
        "DESIREE",
        "ANTONIA",
        "HOPE",
        "GINGER",
        "JANIS",
        "BETSY",
        "CHRISTIE",
        "FREDA",
        "MERCEDES",
        "MEREDITH",
        "LYNETTE",
        "TERI",
        "CRISTINA",
        "EULA",
        "LEIGH",
        "MEGHAN",
        "SOPHIA",
        "ELOISE",
        "ROCHELLE",
        "GRETCHEN",
        "CECELIA",
        "RAQUEL",
        "HENRIETTA",
        "ALYSSA",
        "JANA",
        "KELLEY",
        "GWEN",
        "KERRY",
        "JENNA",
        "TRICIA",
        "LAVERNE",
        "OLIVE",
        "ALEXIS",
        "TASHA",
        "SILVIA",
        "ELVIRA",
        "CASEY",
        "DELIA",
        "SOPHIE",
        "KATE",
        "PATTI",
        "LORENA",
        "KELLIE",
        "SONJA",
        "LILA",
        "LANA",
        "DARLA",
        "MAY",
        "MINDY",
        "ESSIE",
        "MANDY",
        "LORENE",
        "ELSA",
        "JOSEFINA",
        "JEANNIE",
        "MIRANDA",
        "DIXIE",
        "LUCIA",
        "MARTA",
        "FAITH",
        "LELA",
        "JOHANNA",
        "SHARI",
        "CAMILLE",
        "TAMI",
        "SHAWNA",
        "ELISA",
        "EBONY",
        "MELBA",
        "ORA",
        "NETTIE",
        "TABITHA",
        "OLLIE",
        "JAIME",
        "WINIFRED",
        "KRISTIE",
        "MARINA",
        "ALISHA",
        "AIMEE",
        "RENA",
        "MYRNA",
        "MARLA",
        "TAMMIE",
        "LATASHA",
        "BONITA",
        "PATRICE",
        "RONDA",
        "SHERRIE",
        "ADDIE",
        "FRANCINE",
        "DELORIS",
        "STACIE",
        "ADRIANA",
        "CHERI",
        "SHELBY",
        "ABIGAIL",
        "CELESTE",
        "JEWEL",
        "CARA",
        "ADELE",
        "REBEKAH",
        "LUCINDA",
        "DORTHY",
        "CHRIS",
        "EFFIE",
        "TRINA",
        "REBA",
        "SHAWN",
        "SALLIE",
        "AURORA",
        "LENORA",
        "ETTA",
        "LOTTIE",
        "KERRI",
        "TRISHA",
        "NIKKI",
        "ESTELLA",
        "FRANCISCA",
        "JOSIE",
        "TRACIE",
        "MARISSA",
        "KARIN",
        "BRITTNEY",
        "JANELLE",
        "LOURDES",
        "LAUREL",
        "HELENE",
        "FERN",
        "ELVA",
        "CORINNE",
        "KELSEY",
        "INA",
        "BETTIE",
        "ELISABETH",
        "AIDA",
        "CAITLIN",
        "INGRID",
        "IVA",
        "EUGENIA",
        "CHRISTA",
        "GOLDIE",
        "CASSIE",
        "MAUDE",
        "JENIFER",
        "THERESE",
        "FRANKIE",
        "DENA",
        "LORNA",
        "JANETTE",
        "LATONYA",
        "CANDY",
        "MORGAN",
        "CONSUELO",
        "TAMIKA",
        "ROSETTA",
        "DEBORA",
        "CHERIE",
        "POLLY",
        "DINA",
        "JEWELL",
        "FAY",
        "JILLIAN",
        "DOROTHEA",
        "NELL",
        "TRUDY",
        "ESPERANZA",
        "PATRICA",
        "KIMBERLEY",
        "SHANNA",
        "HELENA",
        "CAROLINA",
        "CLEO",
        "STEFANIE",
        "ROSARIO",
        "OLA",
        "JANINE",
        "MOLLIE",
        "LUPE",
        "ALISA",
        "LOU",
        "MARIBEL",
        "SUSANNE",
        "BETTE",
        "SUSANA",
        "ELISE",
        "CECILE",
        "ISABELLE",
        "LESLEY",
        "JOCELYN",
        "PAIGE",
        "JONI",
        "RACHELLE",
        "LEOLA",
        "DAPHNE",
        "ALTA",
        "ESTER",
        "PETRA",
        "GRACIELA",
        "IMOGENE",
        "JOLENE",
        "KEISHA",
        "LACEY",
        "GLENNA",
        "GABRIELA",
        "KERI",
        "URSULA",
        "LIZZIE",
        "KIRSTEN",
        "SHANA",
        "ADELINE",
        "MAYRA",
        "JAYNE",
        "JACLYN",
        "GRACIE",
        "SONDRA",
        "CARMELA",
        "MARISA",
        "ROSALIND",
        "CHARITY",
        "TONIA",
        "BEATRIZ",
        "MARISOL",
        "CLARICE",
        "JEANINE",
        "SHEENA",
        "ANGELINE",
        "FRIEDA",
        "LILY",
        "ROBBIE",
        "SHAUNA",
        "MILLIE",
        "CLAUDETTE",
        "CATHLEEN",
        "ANGELIA",
        "GABRIELLE",
        "AUTUMN",
        "KATHARINE",
        "SUMMER",
        "JODIE",
        "STACI",
        "LEA",
        "CHRISTI",
        "JIMMIE",
        "JUSTINE",
        "ELMA",
        "LUELLA",
        "MARGRET",
        "DOMINIQUE",
        "SOCORRO",
        "RENE",
        "MARTINA",
        "MARGO",
        "MAVIS",
        "CALLIE",
        "BOBBI",
        "MARITZA",
        "LUCILE",
        "LEANNE",
        "JEANNINE",
        "DEANA",
        "AILEEN",
        "LORIE",
        "LADONNA",
        "WILLA",
        "MANUELA",
        "GALE",
        "SELMA",
        "DOLLY",
        "SYBIL",
        "ABBY",
        "LARA",
        "DALE",
        "IVY",
        "DEE",
        "WINNIE",
        "MARCY",
        "LUISA",
        "JERI",
        "MAGDALENA",
        "OFELIA",
        "MEAGAN",
        "AUDRA",
        "MATILDA",
        "LEILA",
        "CORNELIA",
        "BIANCA",
        "SIMONE",
        "BETTYE",
        "RANDI",
        "VIRGIE",
        "LATISHA",
        "BARBRA",
        "GEORGINA",
        "ELIZA",
        "LEANN",
        "BRIDGETTE",
        "RHODA",
        "HALEY",
        "ADELA",
        "NOLA",
        "BERNADINE",
        "FLOSSIE",
        "ILA",
        "GRETA",
        "RUTHIE",
        "NELDA",
        "MINERVA",
        "LILLY",
        "TERRIE",
        "LETHA",
        "HILARY",
        "ESTELA",
        "VALARIE",
        "BRIANNA",
        "ROSALYN",
        "EARLINE",
        "CATALINA",
        "AVA",
        "MIA",
        "CLARISSA",
        "LIDIA",
        "CORRINE",
        "ALEXANDRIA",
        "CONCEPCION",
        "TIA",
        "SHARRON",
        "RAE",
        "DONA",
        "ERICKA",
        "JAMI",
        "ELNORA",
        "CHANDRA",
        "LENORE",
        "NEVA",
        "MARYLOU",
        "MELISA",
        "TABATHA",
        "SERENA",
        "AVIS",
        "ALLIE",
        "SOFIA",
        "JEANIE",
        "ODESSA",
        "NANNIE",
        "HARRIETT",
        "LORAINE",
        "PENELOPE",
        "MILAGROS",
        "EMILIA",
        "BENITA",
        "ALLYSON",
        "ASHLEE",
        "TANIA",
        "TOMMIE",
        "ESMERALDA",
        "KARINA",
        "EVE",
        "PEARLIE",
        "ZELMA",
        "MALINDA",
        "NOREEN",
        "TAMEKA",
        "SAUNDRA",
        "HILLARY",
        "AMIE",
        "ALTHEA",
        "ROSALINDA",
        "JORDAN",
        "LILIA",
        "ALANA",
        "GAY",
        "CLARE",
        "ALEJANDRA",
        "ELINOR",
        "MICHAEL",
        "LORRIE",
        "JERRI",
        "DARCY",
        "EARNESTINE",
        "CARMELLA",
        "TAYLOR",
        "NOEMI",
        "MARCIE",
        "LIZA",
        "ANNABELLE",
        "LOUISA",
        "EARLENE",
        "MALLORY",
        "CARLENE",
        "NITA",
        "SELENA",
        "TANISHA",
        "KATY",
        "JULIANNE",
        "JOHN",
        "LAKISHA",
        "EDWINA",
        "MARICELA",
        "MARGERY",
        "KENYA",
        "DOLLIE",
        "ROXIE",
        "ROSLYN",
        "KATHRINE",
        "NANETTE",
        "CHARMAINE",
        "LAVONNE",
        "ILENE",
        "KRIS",
        "TAMMI",
        "SUZETTE",
        "CORINE",
        "KAYE",
        "JERRY",
        "MERLE",
        "CHRYSTAL",
        "LINA",
        "DEANNE",
        "LILIAN",
        "JULIANA",
        "ALINE",
        "LUANN",
        "KASEY",
        "MARYANNE",
        "EVANGELINE",
        "COLETTE",
        "MELVA",
        "LAWANDA",
        "YESENIA",
        "NADIA",
        "MADGE",
        "KATHIE",
        "EDDIE",
        "OPHELIA",
        "VALERIA",
        "NONA",
        "MITZI",
        "MARI",
        "GEORGETTE",
        "CLAUDINE",
        "FRAN",
        "ALISSA",
        "ROSEANN",
        "LAKEISHA",
        "SUSANNA",
        "REVA",
        "DEIDRE",
        "CHASITY",
        "SHEREE",
        "CARLY",
        "JAMES",
        "ELVIA",
        "ALYCE",
        "DEIRDRE",
        "GENA",
        "BRIANA",
        "ARACELI",
        "KATELYN",
        "ROSANNE",
        "WENDI",
        "TESSA",
        "BERTA",
        "MARVA",
        "IMELDA",
        "MARIETTA",
        "MARCI",
        "LEONOR",
        "ARLINE",
        "SASHA",
        "MADELYN",
        "JANNA",
        "JULIETTE",
        "DEENA",
        "AURELIA",
        "JOSEFA",
        "AUGUSTA",
        "LILIANA",
        "YOUNG",
        "CHRISTIAN",
        "LESSIE",
        "AMALIA",
        "SAVANNAH",
        "ANASTASIA",
        "VILMA",
        "NATALIA",
        "ROSELLA",
        "LYNNETTE",
        "CORINA",
        "ALFREDA",
        "LEANNA",
        "CAREY",
        "AMPARO",
        "COLEEN",
        "TAMRA",
        "AISHA",
        "WILDA",
        "KARYN",
        "CHERRY",
        "QUEEN",
        "MAURA",
        "MAI",
        "EVANGELINA",
        "ROSANNA",
        "HALLIE",
        "ERNA",
        "ENID",
        "MARIANA",
        "LACY",
        "JULIET",
        "JACKLYN",
        "FREIDA",
        "MADELEINE",
        "MARA",
        "HESTER",
        "CATHRYN",
        "LELIA",
        "CASANDRA",
        "BRIDGETT",
        "ANGELITA",
        "JANNIE",
        "DIONNE",
        "ANNMARIE",
        "KATINA",
        "BERYL",
        "PHOEBE",
        "MILLICENT",
        "KATHERYN",
        "DIANN",
        "CARISSA",
        "MARYELLEN",
        "LIZ",
        "LAURI",
        "HELGA",
        "GILDA",
        "ADRIAN",
        "RHEA",
        "MARQUITA",
        "HOLLIE",
        "TISHA",
        "TAMERA",
        "ANGELIQUE",
        "FRANCESCA",
        "BRITNEY",
        "KAITLIN",
        "LOLITA",
        "FLORINE",
        "ROWENA",
        "REYNA",
        "TWILA",
        "FANNY",
        "JANELL",
        "INES",
        "CONCETTA",
        "BERTIE",
        "ALBA",
        "BRIGITTE",
        "ALYSON",
        "VONDA",
        "PANSY",
        "ELBA",
        "NOELLE",
        "LETITIA",
        "KITTY",
        "DEANN",
        "BRANDIE",
        "LOUELLA",
        "LETA",
        "FELECIA",
        "SHARLENE",
        "LESA",
        "BEVERLEY",
        "ROBERT",
        "ISABELLA",
        "HERMINIA",
        "TERRA",
        "CELINA",
    ]

    _male_first_names = [
        "JAMES",
        "JOHN",
        "ROBERT",
        "MICHAEL",
        "WILLIAM",
        "DAVID",
        "RICHARD",
        "CHARLES",
        "JOSEPH",
        "THOMAS",
        "CHRISTOPHER",
        "DANIEL",
        "PAUL",
        "MARK",
        "DONALD",
        "GEORGE",
        "KENNETH",
        "STEVEN",
        "EDWARD",
        "BRIAN",
        "RONALD",
        "ANTHONY",
        "KEVIN",
        "JASON",
        "MATTHEW",
        "GARY",
        "TIMOTHY",
        "JOSE",
        "LARRY",
        "JEFFREY",
        "FRANK",
        "SCOTT",
        "ERIC",
        "STEPHEN",
        "ANDREW",
        "RAYMOND",
        "GREGORY",
        "JOSHUA",
        "JERRY",
        "DENNIS",
        "WALTER",
        "PATRICK",
        "PETER",
        "HAROLD",
        "DOUGLAS",
        "HENRY",
        "CARL",
        "ARTHUR",
        "RYAN",
        "ROGER",
        "JOE",
        "JUAN",
        "JACK",
        "ALBERT",
        "JONATHAN",
        "JUSTIN",
        "TERRY",
        "GERALD",
        "KEITH",
        "SAMUEL",
        "WILLIE",
        "RALPH",
        "LAWRENCE",
        "NICHOLAS",
        "ROY",
        "BENJAMIN",
        "BRUCE",
        "BRANDON",
        "ADAM",
        "HARRY",
        "FRED",
        "WAYNE",
        "BILLY",
        "STEVE",
        "LOUIS",
        "JEREMY",
        "AARON",
        "RANDY",
        "HOWARD",
        "EUGENE",
        "CARLOS",
        "RUSSELL",
        "BOBBY",
        "VICTOR",
        "MARTIN",
        "ERNEST",
        "PHILLIP",
        "TODD",
        "JESSE",
        "CRAIG",
        "ALAN",
        "SHAWN",
        "CLARENCE",
        "SEAN",
        "PHILIP",
        "CHRIS",
        "JOHNNY",
        "EARL",
        "JIMMY",
        "ANTONIO",
        "DANNY",
        "BRYAN",
        "TONY",
        "LUIS",
        "MIKE",
        "STANLEY",
        "LEONARD",
        "NATHAN",
        "DALE",
        "MANUEL",
        "RODNEY",
        "CURTIS",
        "NORMAN",
        "ALLEN",
        "MARVIN",
        "VINCENT",
        "GLENN",
        "JEFFERY",
        "TRAVIS",
        "JEFF",
        "CHAD",
        "JACOB",
        "LEE",
        "MELVIN",
        "ALFRED",
        "KYLE",
        "FRANCIS",
        "BRADLEY",
        "JESUS",
        "HERBERT",
        "FREDERICK",
        "RAY",
        "JOEL",
        "EDWIN",
        "DON",
        "EDDIE",
        "RICKY",
        "TROY",
        "RANDALL",
        "BARRY",
        "ALEXANDER",
        "BERNARD",
        "MARIO",
        "LEROY",
        "FRANCISCO",
        "MARCUS",
        "MICHEAL",
        "THEODORE",
        "CLIFFORD",
        "MIGUEL",
        "OSCAR",
        "JAY",
        "JIM",
        "TOM",
        "CALVIN",
        "ALEX",
        "JON",
        "RONNIE",
        "BILL",
        "LLOYD",
        "TOMMY",
        "LEON",
        "DEREK",
        "WARREN",
        "DARRELL",
        "JEROME",
        "FLOYD",
        "LEO",
        "ALVIN",
        "TIM",
        "WESLEY",
        "GORDON",
        "DEAN",
        "GREG",
        "JORGE",
        "DUSTIN",
        "PEDRO",
        "DERRICK",
        "DAN",
        "LEWIS",
        "ZACHARY",
        "COREY",
        "HERMAN",
        "MAURICE",
        "VERNON",
        "ROBERTO",
        "CLYDE",
        "GLEN",
        "HECTOR",
        "SHANE",
        "RICARDO",
        "SAM",
        "RICK",
        "LESTER",
        "BRENT",
        "RAMON",
        "CHARLIE",
        "TYLER",
        "GILBERT",
        "GENE",
        "MARC",
        "REGINALD",
        "RUBEN",
        "BRETT",
        "ANGEL",
        "NATHANIEL",
        "RAFAEL",
        "LESLIE",
        "EDGAR",
        "MILTON",
        "RAUL",
        "BEN",
        "CHESTER",
        "CECIL",
        "DUANE",
        "FRANKLIN",
        "ANDRE",
        "ELMER",
        "BRAD",
        "GABRIEL",
        "RON",
        "MITCHELL",
        "ROLAND",
        "ARNOLD",
        "HARVEY",
        "JARED",
        "ADRIAN",
        "KARL",
        "CORY",
        "CLAUDE",
        "ERIK",
        "DARRYL",
        "JAMIE",
        "NEIL",
        "JESSIE",
        "CHRISTIAN",
        "JAVIER",
        "FERNANDO",
        "CLINTON",
        "TED",
        "MATHEW",
        "TYRONE",
        "DARREN",
        "LONNIE",
        "LANCE",
        "CODY",
        "JULIO",
        "KELLY",
        "KURT",
        "ALLAN",
        "NELSON",
        "GUY",
        "CLAYTON",
        "HUGH",
        "MAX",
        "DWAYNE",
        "DWIGHT",
        "ARMANDO",
        "FELIX",
        "JIMMIE",
        "EVERETT",
        "JORDAN",
        "IAN",
        "WALLACE",
        "KEN",
        "BOB",
        "JAIME",
        "CASEY",
        "ALFREDO",
        "ALBERTO",
        "DAVE",
        "IVAN",
        "JOHNNIE",
        "SIDNEY",
        "BYRON",
        "JULIAN",
        "ISAAC",
        "MORRIS",
        "CLIFTON",
        "WILLARD",
        "DARYL",
        "ROSS",
        "VIRGIL",
        "ANDY",
        "MARSHALL",
        "SALVADOR",
        "PERRY",
        "KIRK",
        "SERGIO",
        "MARION",
        "TRACY",
        "SETH",
        "KENT",
        "TERRANCE",
        "RENE",
        "EDUARDO",
        "TERRENCE",
        "ENRIQUE",
        "FREDDIE",
        "WADE",
        "AUSTIN",
        "STUART",
        "FREDRICK",
        "ARTURO",
        "ALEJANDRO",
        "JACKIE",
        "JOEY",
        "NICK",
        "LUTHER",
        "WENDELL",
        "JEREMIAH",
        "EVAN",
        "JULIUS",
        "DANA",
        "DONNIE",
        "OTIS",
        "SHANNON",
        "TREVOR",
        "OLIVER",
        "LUKE",
        "HOMER",
        "GERARD",
        "DOUG",
        "KENNY",
        "HUBERT",
        "ANGELO",
        "SHAUN",
        "LYLE",
        "MATT",
        "LYNN",
        "ALFONSO",
        "ORLANDO",
        "REX",
        "CARLTON",
        "ERNESTO",
        "CAMERON",
        "NEAL",
        "PABLO",
        "LORENZO",
        "OMAR",
        "WILBUR",
        "BLAKE",
        "GRANT",
        "HORACE",
        "RODERICK",
        "KERRY",
        "ABRAHAM",
        "WILLIS",
        "RICKEY",
        "JEAN",
        "IRA",
        "ANDRES",
        "CESAR",
        "JOHNATHAN",
        "MALCOLM",
        "RUDOLPH",
        "DAMON",
        "KELVIN",
        "RUDY",
        "PRESTON",
        "ALTON",
        "ARCHIE",
        "MARCO",
        "WM",
        "PETE",
        "RANDOLPH",
        "GARRY",
        "GEOFFREY",
        "JONATHON",
        "FELIPE",
        "BENNIE",
        "GERARDO",
        "ED",
        "DOMINIC",
        "ROBIN",
        "LOREN",
        "DELBERT",
        "COLIN",
        "GUILLERMO",
        "EARNEST",
        "LUCAS",
        "BENNY",
        "NOEL",
        "SPENCER",
        "RODOLFO",
        "MYRON",
        "EDMUND",
        "GARRETT",
        "SALVATORE",
        "CEDRIC",
        "LOWELL",
        "GREGG",
        "SHERMAN",
        "WILSON",
        "DEVIN",
        "SYLVESTER",
        "KIM",
        "ROOSEVELT",
        "ISRAEL",
        "JERMAINE",
        "FORREST",
        "WILBERT",
        "LELAND",
        "SIMON",
        "GUADALUPE",
        "CLARK",
        "IRVING",
        "CARROLL",
        "BRYANT",
        "OWEN",
        "RUFUS",
        "WOODROW",
        "SAMMY",
        "KRISTOPHER",
        "MACK",
        "LEVI",
        "MARCOS",
        "GUSTAVO",
        "JAKE",
        "LIONEL",
        "MARTY",
        "TAYLOR",
        "ELLIS",
        "DALLAS",
        "GILBERTO",
        "CLINT",
        "NICOLAS",
        "LAURENCE",
        "ISMAEL",
        "ORVILLE",
        "DREW",
        "JODY",
        "ERVIN",
        "DEWEY",
        "AL",
        "WILFRED",
        "JOSH",
        "HUGO",
        "IGNACIO",
        "CALEB",
        "TOMAS",
        "SHELDON",
        "ERICK",
        "FRANKIE",
        "STEWART",
        "DOYLE",
        "DARREL",
        "ROGELIO",
        "TERENCE",
        "SANTIAGO",
        "ALONZO",
        "ELIAS",
        "BERT",
        "ELBERT",
        "RAMIRO",
        "CONRAD",
        "PAT",
        "NOAH",
        "GRADY",
        "PHIL",
        "CORNELIUS",
        "LAMAR",
        "ROLANDO",
        "CLAY",
        "PERCY",
        "DEXTER",
        "BRADFORD",
        "MERLE",
        "DARIN",
        "AMOS",
        "TERRELL",
        "MOSES",
        "IRVIN",
        "SAUL",
        "ROMAN",
        "DARNELL",
        "RANDAL",
        "TOMMIE",
        "TIMMY",
        "DARRIN",
        "WINSTON",
        "BRENDAN",
        "TOBY",
        "VAN",
        "ABEL",
        "DOMINICK",
        "BOYD",
        "COURTNEY",
        "JAN",
        "EMILIO",
        "ELIJAH",
        "CARY",
        "DOMINGO",
        "SANTOS",
        "AUBREY",
        "EMMETT",
        "MARLON",
        "EMANUEL",
        "JERALD",
        "EDMOND",
        "EMIL",
        "DEWAYNE",
        "WILL",
        "OTTO",
        "TEDDY",
        "REYNALDO",
        "BRET",
        "MORGAN",
        "JESS",
        "TRENT",
        "HUMBERTO",
        "EMMANUEL",
        "STEPHAN",
        "LOUIE",
        "VICENTE",
        "LAMONT",
        "STACY",
        "GARLAND",
        "MILES",
        "MICAH",
        "EFRAIN",
        "BILLIE",
        "LOGAN",
        "HEATH",
        "RODGER",
        "HARLEY",
        "DEMETRIUS",
        "ETHAN",
        "ELDON",
        "ROCKY",
        "PIERRE",
        "JUNIOR",
        "FREDDY",
        "ELI",
        "BRYCE",
        "ANTOINE",
        "ROBBIE",
        "KENDALL",
        "ROYCE",
        "STERLING",
        "MICKEY",
        "CHASE",
        "GROVER",
        "ELTON",
        "CLEVELAND",
        "DYLAN",
        "CHUCK",
        "DAMIAN",
        "REUBEN",
        "STAN",
        "AUGUST",
        "LEONARDO",
        "JASPER",
        "RUSSEL",
        "ERWIN",
        "BENITO",
        "HANS",
        "MONTE",
        "BLAINE",
        "ERNIE",
        "CURT",
        "QUENTIN",
        "AGUSTIN",
        "MURRAY",
        "JAMAL",
        "DEVON",
        "ADOLFO",
        "HARRISON",
        "TYSON",
        "BURTON",
        "BRADY",
        "ELLIOTT",
        "WILFREDO",
        "BART",
        "JARROD",
        "VANCE",
        "DENIS",
        "DAMIEN",
        "JOAQUIN",
        "HARLAN",
        "DESMOND",
        "ELLIOT",
        "DARWIN",
        "ASHLEY",
        "GREGORIO",
        "BUDDY",
        "XAVIER",
        "KERMIT",
        "ROSCOE",
        "ESTEBAN",
        "ANTON",
        "SOLOMON",
        "SCOTTY",
        "NORBERT",
        "ELVIN",
        "WILLIAMS",
        "NOLAN",
        "CAREY",
        "ROD",
        "QUINTON",
        "HAL",
        "BRAIN",
        "ROB",
        "ELWOOD",
        "KENDRICK",
        "DARIUS",
        "MOISES",
        "SON",
        "MARLIN",
        "FIDEL",
        "THADDEUS",
        "CLIFF",
        "MARCEL",
        "ALI",
        "JACKSON",
        "RAPHAEL",
        "BRYON",
        "ARMAND",
        "ALVARO",
        "JEFFRY",
        "DANE",
        "JOESPH",
        "THURMAN",
        "NED",
        "SAMMIE",
        "RUSTY",
        "MICHEL",
        "MONTY",
        "RORY",
        "FABIAN",
        "REGGIE",
        "MASON",
        "GRAHAM",
        "KRIS",
        "ISAIAH",
        "VAUGHN",
        "GUS",
        "AVERY",
        "LOYD",
        "DIEGO",
        "ALEXIS",
        "ADOLPH",
        "NORRIS",
        "MILLARD",
        "ROCCO",
        "GONZALO",
        "DERICK",
        "RODRIGO",
        "GERRY",
        "STACEY",
        "CARMEN",
        "WILEY",
        "RIGOBERTO",
        "ALPHONSO",
        "TY",
        "SHELBY",
        "RICKIE",
        "NOE",
        "VERN",
        "BOBBIE",
        "REED",
        "JEFFERSON",
        "ELVIS",
        "BERNARDO",
        "MAURICIO",
        "HIRAM",
        "DONOVAN",
        "BASIL",
        "RILEY",
        "OLLIE",
        "NICKOLAS",
        "MAYNARD",
        "SCOT",
        "VINCE",
        "QUINCY",
        "EDDY",
        "SEBASTIAN",
        "FEDERICO",
        "ULYSSES",
        "HERIBERTO",
        "DONNELL",
        "COLE",
        "DENNY",
        "DAVIS",
        "GAVIN",
        "EMERY",
        "WARD",
        "ROMEO",
        "JAYSON",
        "DION",
        "DANTE",
        "CLEMENT",
        "COY",
        "ODELL",
        "MAXWELL",
        "JARVIS",
        "BRUNO",
        "ISSAC",
        "MARY",
        "DUDLEY",
        "BROCK",
        "SANFORD",
        "COLBY",
        "CARMELO",
        "BARNEY",
        "NESTOR",
        "HOLLIS",
        "STEFAN",
        "DONNY",
        "ART",
        "LINWOOD",
        "BEAU",
        "WELDON",
        "GALEN",
        "ISIDRO",
        "TRUMAN",
        "DELMAR",
        "JOHNATHON",
        "SILAS",
        "FREDERIC",
        "DICK",
        "KIRBY",
        "IRWIN",
        "CRUZ",
        "MERLIN",
        "MERRILL",
        "CHARLEY",
        "MARCELINO",
        "LANE",
        "HARRIS",
        "CLEO",
        "CARLO",
        "TRENTON",
        "KURTIS",
        "HUNTER",
        "AURELIO",
        "WINFRED",
        "VITO",
        "COLLIN",
        "DENVER",
        "CARTER",
        "LEONEL",
        "EMORY",
        "PASQUALE",
        "MOHAMMAD",
        "MARIANO",
        "DANIAL",
        "BLAIR",
        "LANDON",
        "DIRK",
        "BRANDEN",
        "ADAN",
        "NUMBERS",
        "CLAIR",
        "BUFORD",
        "GERMAN",
        "BERNIE",
        "WILMER",
        "JOAN",
        "EMERSON",
        "ZACHERY",
        "FLETCHER",
        "JACQUES",
        "ERROL",
        "DALTON",
        "MONROE",
        "JOSUE",
        "DOMINIQUE",
        "EDWARDO",
        "BOOKER",
        "WILFORD",
        "SONNY",
        "SHELTON",
        "CARSON",
        "THERON",
        "RAYMUNDO",
        "DAREN",
        "TRISTAN",
        "HOUSTON",
        "ROBBY",
        "LINCOLN",
        "JAME",
        "GENARO",
        "GALE",
        "BENNETT",
        "OCTAVIO",
        "CORNELL",
        "LAVERNE",
        "HUNG",
        "ARRON",
        "ANTONY",
        "HERSCHEL",
        "ALVA",
        "GIOVANNI",
        "GARTH",
        "CYRUS",
        "CYRIL",
        "RONNY",
        "STEVIE",
        "LON",
        "FREEMAN",
        "ERIN",
        "DUNCAN",
        "KENNITH",
        "CARMINE",
        "AUGUSTINE",
        "YOUNG",
        "ERICH",
        "CHADWICK",
        "WILBURN",
        "RUSS",
        "REID",
        "MYLES",
        "ANDERSON",
        "MORTON",
        "JONAS",
        "FOREST",
        "MITCHEL",
        "MERVIN",
        "ZANE",
        "RICH",
        "JAMEL",
        "LAZARO",
        "ALPHONSE",
        "RANDELL",
        "MAJOR",
        "JOHNIE",
        "JARRETT",
        "BROOKS",
        "ARIEL",
        "ABDUL",
        "DUSTY",
        "LUCIANO",
        "LINDSEY",
        "TRACEY",
        "SEYMOUR",
        "SCOTTIE",
        "EUGENIO",
        "MOHAMMED",
        "SANDY",
        "VALENTIN",
        "CHANCE",
        "ARNULFO",
        "LUCIEN",
        "FERDINAND",
        "THAD",
        "EZRA",
        "SYDNEY",
        "ALDO",
        "RUBIN",
        "ROYAL",
        "MITCH",
        "EARLE",
        "ABE",
        "WYATT",
        "MARQUIS",
        "LANNY",
        "KAREEM",
        "JAMAR",
        "BORIS",
        "ISIAH",
        "EMILE",
        "ELMO",
        "ARON",
        "LEOPOLDO",
        "EVERETTE",
        "JOSEF",
        "GAIL",
        "ELOY",
        "DORIAN",
        "RODRICK",
        "REINALDO",
        "LUCIO",
        "JERROD",
        "WESTON",
        "HERSHEL",
        "BARTON",
        "PARKER",
        "LEMUEL",
        "LAVERN",
        "BURT",
        "JULES",
        "GIL",
        "ELISEO",
        "AHMAD",
        "NIGEL",
        "EFREN",
        "ANTWAN",
        "ALDEN",
        "MARGARITO",
        "COLEMAN",
        "REFUGIO",
        "DINO",
        "OSVALDO",
        "LES",
        "DEANDRE",
        "NORMAND",
        "KIETH",
        "IVORY",
        "ANDREA",
        "TREY",
        "NORBERTO",
        "NAPOLEON",
        "JEROLD",
        "FRITZ",
        "ROSENDO",
        "MILFORD",
        "SANG",
        "DEON",
        "CHRISTOPER",
        "ALFONZO",
        "LYMAN",
        "JOSIAH",
        "BRANT",
        "WILTON",
        "RICO",
        "JAMAAL",
        "DEWITT",
        "CAROL",
        "BRENTON",
        "YONG",
        "OLIN",
        "FOSTER",
        "FAUSTINO",
        "CLAUDIO",
        "JUDSON",
        "GINO",
        "EDGARDO",
        "BERRY",
        "ALEC",
        "TANNER",
        "JARRED",
        "DONN",
        "TRINIDAD",
        "TAD",
        "SHIRLEY",
        "PRINCE",
        "PORFIRIO",
        "ODIS",
        "MARIA",
        "LENARD",
        "CHAUNCEY",
        "CHANG",
        "TOD",
        "MEL",
        "MARCELO",
        "KORY",
        "AUGUSTUS",
        "KEVEN",
        "HILARIO",
        "BUD",
        "SAL",
        "ROSARIO",
        "ORVAL",
        "MAURO",
        "DANNIE",
        "ZACHARIAH",
        "OLEN",
        "ANIBAL",
        "MILO",
        "JED",
        "FRANCES",
        "THANH",
        "DILLON",
        "AMADO",
        "NEWTON",
        "CONNIE",
        "LENNY",
        "TORY",
        "RICHIE",
        "LUPE",
        "HORACIO",
        "BRICE",
        "MOHAMED",
        "DELMER",
        "DARIO",
        "REYES",
        "DEE",
        "MAC",
        "JONAH",
        "JERROLD",
        "ROBT",
        "HANK",
        "SUNG",
        "RUPERT",
        "ROLLAND",
        "KENTON",
        "DAMION",
        "CHI",
        "ANTONE",
        "WALDO",
        "FREDRIC",
        "BRADLY",
        "QUINN",
        "KIP",
        "BURL",
        "WALKER",
        "TYREE",
        "JEFFEREY",
        "AHMED",
    ]

    _last_names = [
        "SMITH",
        "JOHNSON",
        "WILLIAMS",
        "JONES",
        "BROWN",
        "DAVIS",
        "MILLER",
        "WILSON",
        "MOORE",
        "TAYLOR",
        "ANDERSON",
        "THOMAS",
        "JACKSON",
        "WHITE",
        "HARRIS",
        "MARTIN",
        "THOMPSON",
        "GARCIA",
        "MARTINEZ",
        "ROBINSON",
        "CLARK",
        "RODRIGUEZ",
        "LEWIS",
        "LEE",
        "WALKER",
        "HALL",
        "ALLEN",
        "YOUNG",
        "HERNANDEZ",
        "KING",
        "WRIGHT",
        "LOPEZ",
        "HILL",
        "SCOTT",
        "GREEN",
        "ADAMS",
        "BAKER",
        "GONZALEZ",
        "NELSON",
        "CARTER",
        "MITCHELL",
        "PEREZ",
        "ROBERTS",
        "TURNER",
        "PHILLIPS",
        "CAMPBELL",
        "PARKER",
        "EVANS",
        "EDWARDS",
        "COLLINS",
        "STEWART",
        "SANCHEZ",
        "MORRIS",
        "ROGERS",
        "REED",
        "COOK",
        "MORGAN",
        "BELL",
        "MURPHY",
        "BAILEY",
        "RIVERA",
        "COOPER",
        "RICHARDSON",
        "COX",
        "HOWARD",
        "WARD",
        "TORRES",
        "PETERSON",
        "GRAY",
        "RAMIREZ",
        "JAMES",
        "WATSON",
        "BROOKS",
        "KELLY",
        "SANDERS",
        "PRICE",
        "BENNETT",
        "WOOD",
        "BARNES",
        "ROSS",
        "HENDERSON",
        "COLEMAN",
        "JENKINS",
        "PERRY",
        "POWELL",
        "LONG",
        "PATTERSON",
        "HUGHES",
        "FLORES",
        "WASHINGTON",
        "BUTLER",
        "SIMMONS",
        "FOSTER",
        "GONZALES",
        "BRYANT",
        "ALEXANDER",
        "RUSSELL",
        "GRIFFIN",
        "DIAZ",
        "HAYES",
        "MYERS",
        "FORD",
        "HAMILTON",
        "GRAHAM",
        "SULLIVAN",
        "WALLACE",
        "WOODS",
        "COLE",
        "WEST",
        "JORDAN",
        "OWENS",
        "REYNOLDS",
        "FISHER",
        "ELLIS",
        "HARRISON",
        "GIBSON",
        "MCDONALD",
        "CRUZ",
        "MARSHALL",
        "ORTIZ",
        "GOMEZ",
        "MURRAY",
        "FREEMAN",
        "WELLS",
        "WEBB",
        "SIMPSON",
        "STEVENS",
        "TUCKER",
        "PORTER",
        "HUNTER",
        "HICKS",
        "CRAWFORD",
        "HENRY",
        "BOYD",
        "MASON",
        "MORALES",
        "KENNEDY",
        "WARREN",
        "DIXON",
        "RAMOS",
        "REYES",
        "BURNS",
        "GORDON",
        "SHAW",
        "HOLMES",
        "RICE",
        "ROBERTSON",
        "HUNT",
        "BLACK",
        "DANIELS",
        "PALMER",
        "MILLS",
        "NICHOLS",
        "GRANT",
        "KNIGHT",
        "FERGUSON",
        "ROSE",
        "STONE",
        "HAWKINS",
        "DUNN",
        "PERKINS",
        "HUDSON",
        "SPENCER",
        "GARDNER",
        "STEPHENS",
        "PAYNE",
        "PIERCE",
        "BERRY",
        "MATTHEWS",
        "ARNOLD",
        "WAGNER",
        "WILLIS",
        "RAY",
        "WATKINS",
        "OLSON",
        "CARROLL",
        "DUNCAN",
        "SNYDER",
        "HART",
        "CUNNINGHAM",
        "BRADLEY",
        "LANE",
        "ANDREWS",
        "RUIZ",
        "HARPER",
        "FOX",
        "RILEY",
        "ARMSTRONG",
        "CARPENTER",
        "WEAVER",
        "GREENE",
        "LAWRENCE",
        "ELLIOTT",
        "CHAVEZ",
        "SIMS",
        "AUSTIN",
        "PETERS",
        "KELLEY",
        "FRANKLIN",
        "LAWSON",
        "FIELDS",
        "GUTIERREZ",
        "RYAN",
        "SCHMIDT",
        "CARR",
        "VASQUEZ",
        "CASTILLO",
        "WHEELER",
        "CHAPMAN",
        "OLIVER",
        "MONTGOMERY",
        "RICHARDS",
        "WILLIAMSON",
        "JOHNSTON",
        "BANKS",
        "MEYER",
        "BISHOP",
        "MCCOY",
        "HOWELL",
        "ALVAREZ",
        "MORRISON",
        "HANSEN",
        "FERNANDEZ",
        "GARZA",
        "HARVEY",
        "LITTLE",
        "BURTON",
        "STANLEY",
        "NGUYEN",
        "GEORGE",
        "JACOBS",
        "REID",
        "KIM",
        "FULLER",
        "LYNCH",
        "DEAN",
        "GILBERT",
        "GARRETT",
        "ROMERO",
        "WELCH",
        "LARSON",
        "FRAZIER",
        "BURKE",
        "HANSON",
        "DAY",
        "MENDOZA",
        "MORENO",
        "BOWMAN",
        "MEDINA",
        "FOWLER",
        "BREWER",
        "HOFFMAN",
        "CARLSON",
        "SILVA",
        "PEARSON",
        "HOLLAND",
        "DOUGLAS",
        "FLEMING",
        "JENSEN",
        "VARGAS",
        "BYRD",
        "DAVIDSON",
        "HOPKINS",
        "MAY",
        "TERRY",
        "HERRERA",
        "WADE",
        "SOTO",
        "WALTERS",
        "CURTIS",
        "NEAL",
        "CALDWELL",
        "LOWE",
        "JENNINGS",
        "BARNETT",
        "GRAVES",
        "JIMENEZ",
        "HORTON",
        "SHELTON",
        "BARRETT",
        "OBRIEN",
        "CASTRO",
        "SUTTON",
        "GREGORY",
        "MCKINNEY",
        "LUCAS",
        "MILES",
        "CRAIG",
        "RODRIQUEZ",
        "CHAMBERS",
        "HOLT",
        "LAMBERT",
        "FLETCHER",
        "WATTS",
        "BATES",
        "HALE",
        "RHODES",
        "PENA",
        "BECK",
        "NEWMAN",
        "HAYNES",
        "MCDANIEL",
        "MENDEZ",
        "BUSH",
        "VAUGHN",
        "PARKS",
        "DAWSON",
        "SANTIAGO",
        "NORRIS",
        "HARDY",
        "LOVE",
        "STEELE",
        "CURRY",
        "POWERS",
        "SCHULTZ",
        "BARKER",
        "GUZMAN",
        "PAGE",
        "MUNOZ",
        "BALL",
        "KELLER",
        "CHANDLER",
        "WEBER",
        "LEONARD",
        "WALSH",
        "LYONS",
        "RAMSEY",
        "WOLFE",
        "SCHNEIDER",
        "MULLINS",
        "BENSON",
        "SHARP",
        "BOWEN",
        "DANIEL",
        "BARBER",
        "CUMMINGS",
        "HINES",
        "BALDWIN",
        "GRIFFITH",
        "VALDEZ",
        "HUBBARD",
        "SALAZAR",
        "REEVES",
        "WARNER",
        "STEVENSON",
        "BURGESS",
        "SANTOS",
        "TATE",
        "CROSS",
        "GARNER",
        "MANN",
        "MACK",
        "MOSS",
        "THORNTON",
        "DENNIS",
        "MCGEE",
        "FARMER",
        "DELGADO",
        "AGUILAR",
        "VEGA",
        "GLOVER",
        "MANNING",
        "COHEN",
        "HARMON",
        "RODGERS",
        "ROBBINS",
        "NEWTON",
        "TODD",
        "BLAIR",
        "HIGGINS",
        "INGRAM",
        "REESE",
        "CANNON",
        "STRICKLAND",
        "TOWNSEND",
        "POTTER",
        "GOODWIN",
        "WALTON",
        "ROWE",
        "HAMPTON",
        "ORTEGA",
        "PATTON",
        "SWANSON",
        "JOSEPH",
        "FRANCIS",
        "GOODMAN",
        "MALDONADO",
        "YATES",
        "BECKER",
        "ERICKSON",
        "HODGES",
        "RIOS",
        "CONNER",
        "ADKINS",
        "WEBSTER",
        "NORMAN",
        "MALONE",
        "HAMMOND",
        "FLOWERS",
        "COBB",
        "MOODY",
        "QUINN",
        "BLAKE",
        "MAXWELL",
        "POPE",
        "FLOYD",
        "OSBORNE",
        "PAUL",
        "MCCARTHY",
        "GUERRERO",
        "LINDSEY",
        "ESTRADA",
        "SANDOVAL",
        "GIBBS",
        "TYLER",
        "GROSS",
        "FITZGERALD",
        "STOKES",
        "DOYLE",
        "SHERMAN",
        "SAUNDERS",
        "WISE",
        "COLON",
        "GILL",
        "ALVARADO",
        "GREER",
        "PADILLA",
        "SIMON",
        "WATERS",
        "NUNEZ",
        "BALLARD",
        "SCHWARTZ",
        "MCBRIDE",
        "HOUSTON",
        "CHRISTENSEN",
        "KLEIN",
        "PRATT",
        "BRIGGS",
        "PARSONS",
        "MCLAUGHLIN",
        "ZIMMERMAN",
        "FRENCH",
        "BUCHANAN",
        "MORAN",
        "COPELAND",
        "ROY",
        "PITTMAN",
        "BRADY",
        "MCCORMICK",
        "HOLLOWAY",
        "BROCK",
        "POOLE",
        "FRANK",
        "LOGAN",
        "OWEN",
        "BASS",
        "MARSH",
        "DRAKE",
        "WONG",
        "JEFFERSON",
        "PARK",
        "MORTON",
        "ABBOTT",
        "SPARKS",
        "PATRICK",
        "NORTON",
        "HUFF",
        "CLAYTON",
        "MASSEY",
        "LLOYD",
        "FIGUEROA",
        "CARSON",
        "BOWERS",
        "ROBERSON",
        "BARTON",
        "TRAN",
        "LAMB",
        "HARRINGTON",
        "CASEY",
        "BOONE",
        "CORTEZ",
        "CLARKE",
        "MATHIS",
        "SINGLETON",
        "WILKINS",
        "CAIN",
        "BRYAN",
        "UNDERWOOD",
        "HOGAN",
        "MCKENZIE",
        "COLLIER",
        "LUNA",
        "PHELPS",
        "MCGUIRE",
        "ALLISON",
        "BRIDGES",
        "WILKERSON",
        "NASH",
        "SUMMERS",
        "ATKINS",
        "WILCOX",
        "PITTS",
        "CONLEY",
        "MARQUEZ",
        "BURNETT",
        "RICHARD",
        "COCHRAN",
        "CHASE",
        "DAVENPORT",
        "HOOD",
        "GATES",
        "CLAY",
        "AYALA",
        "SAWYER",
        "ROMAN",
        "VAZQUEZ",
        "DICKERSON",
        "HODGE",
        "ACOSTA",
        "FLYNN",
        "ESPINOZA",
        "NICHOLSON",
        "MONROE",
        "WOLF",
        "MORROW",
        "KIRK",
        "RANDALL",
        "ANTHONY",
        "WHITAKER",
        "OCONNOR",
        "SKINNER",
        "WARE",
        "MOLINA",
        "KIRBY",
        "HUFFMAN",
        "BRADFORD",
        "CHARLES",
        "GILMORE",
        "DOMINGUEZ",
        "ONEAL",
        "BRUCE",
        "LANG",
        "COMBS",
        "KRAMER",
        "HEATH",
        "HANCOCK",
        "GALLAGHER",
        "GAINES",
        "SHAFFER",
        "SHORT",
        "WIGGINS",
        "MATHEWS",
        "MCCLAIN",
        "FISCHER",
        "WALL",
        "SMALL",
        "MELTON",
        "HENSLEY",
        "BOND",
        "DYER",
        "CAMERON",
        "GRIMES",
        "CONTRERAS",
        "CHRISTIAN",
        "WYATT",
        "BAXTER",
        "SNOW",
        "MOSLEY",
        "SHEPHERD",
        "LARSEN",
        "HOOVER",
        "BEASLEY",
        "GLENN",
        "PETERSEN",
        "WHITEHEAD",
        "MEYERS",
        "KEITH",
        "GARRISON",
        "VINCENT",
        "SHIELDS",
        "HORN",
        "SAVAGE",
        "OLSEN",
        "SCHROEDER",
        "HARTMAN",
        "WOODARD",
        "MUELLER",
        "KEMP",
        "DELEON",
        "BOOTH",
        "PATEL",
        "CALHOUN",
        "WILEY",
        "EATON",
        "CLINE",
        "NAVARRO",
        "HARRELL",
        "LESTER",
        "HUMPHREY",
        "PARRISH",
        "DURAN",
        "HUTCHINSON",
        "HESS",
        "DORSEY",
        "BULLOCK",
        "ROBLES",
        "BEARD",
        "DALTON",
        "AVILA",
        "VANCE",
        "RICH",
        "BLACKWELL",
        "YORK",
        "JOHNS",
        "BLANKENSHIP",
        "TREVINO",
        "SALINAS",
        "CAMPOS",
        "PRUITT",
        "MOSES",
        "CALLAHAN",
        "GOLDEN",
        "MONTOYA",
        "HARDIN",
        "GUERRA",
        "MCDOWELL",
        "CAREY",
        "STAFFORD",
        "GALLEGOS",
        "HENSON",
        "WILKINSON",
        "BOOKER",
        "MERRITT",
        "MIRANDA",
        "ATKINSON",
        "ORR",
        "DECKER",
        "HOBBS",
        "PRESTON",
        "TANNER",
        "KNOX",
        "PACHECO",
        "STEPHENSON",
        "GLASS",
        "ROJAS",
        "SERRANO",
        "MARKS",
        "HICKMAN",
        "ENGLISH",
        "SWEENEY",
        "STRONG",
        "PRINCE",
        "MCCLURE",
        "CONWAY",
        "WALTER",
        "ROTH",
        "MAYNARD",
        "FARRELL",
        "LOWERY",
        "HURST",
        "NIXON",
        "WEISS",
        "TRUJILLO",
        "ELLISON",
        "SLOAN",
        "JUAREZ",
        "WINTERS",
        "MCLEAN",
        "RANDOLPH",
        "LEON",
        "BOYER",
        "VILLARREAL",
        "MCCALL",
        "GENTRY",
        "CARRILLO",
        "KENT",
        "AYERS",
        "LARA",
        "SHANNON",
        "SEXTON",
        "PACE",
        "HULL",
        "LEBLANC",
        "BROWNING",
        "VELASQUEZ",
        "LEACH",
        "CHANG",
        "HOUSE",
        "SELLERS",
        "HERRING",
        "NOBLE",
        "FOLEY",
        "BARTLETT",
        "MERCADO",
        "LANDRY",
        "DURHAM",
        "WALLS",
        "BARR",
        "MCKEE",
        "BAUER",
        "RIVERS",
        "EVERETT",
        "BRADSHAW",
        "PUGH",
        "VELEZ",
        "RUSH",
        "ESTES",
        "DODSON",
        "MORSE",
        "SHEPPARD",
        "WEEKS",
        "CAMACHO",
        "BEAN",
        "BARRON",
        "LIVINGSTON",
        "MIDDLETON",
        "SPEARS",
        "BRANCH",
        "BLEVINS",
        "CHEN",
        "KERR",
        "MCCONNELL",
        "HATFIELD",
        "HARDING",
        "ASHLEY",
        "SOLIS",
        "HERMAN",
        "FROST",
        "GILES",
        "BLACKBURN",
        "WILLIAM",
        "PENNINGTON",
        "WOODWARD",
        "FINLEY",
        "MCINTOSH",
        "KOCH",
        "BEST",
        "SOLOMON",
        "MCCULLOUGH",
        "DUDLEY",
        "NOLAN",
        "BLANCHARD",
        "RIVAS",
        "BRENNAN",
        "MEJIA",
        "KANE",
        "BENTON",
        "JOYCE",
        "BUCKLEY",
        "HALEY",
        "VALENTINE",
        "MADDOX",
        "RUSSO",
        "MCKNIGHT",
        "BUCK",
        "MOON",
        "MCMILLAN",
        "CROSBY",
        "BERG",
        "DOTSON",
        "MAYS",
        "ROACH",
        "CHURCH",
        "CHAN",
        "RICHMOND",
        "MEADOWS",
        "FAULKNER",
        "ONEILL",
        "KNAPP",
        "KLINE",
        "BARRY",
        "OCHOA",
        "JACOBSON",
        "GAY",
        "AVERY",
        "HENDRICKS",
        "HORNE",
        "SHEPARD",
        "HEBERT",
        "CHERRY",
        "CARDENAS",
        "MCINTYRE",
        "WHITNEY",
        "WALLER",
        "HOLMAN",
        "DONALDSON",
        "CANTU",
        "TERRELL",
        "MORIN",
        "GILLESPIE",
        "FUENTES",
        "TILLMAN",
        "SANFORD",
        "BENTLEY",
        "PECK",
        "KEY",
        "SALAS",
        "ROLLINS",
        "GAMBLE",
        "DICKSON",
        "BATTLE",
        "SANTANA",
        "CABRERA",
        "CERVANTES",
        "HOWE",
        "HINTON",
        "HURLEY",
        "SPENCE",
        "ZAMORA",
        "YANG",
        "MCNEIL",
        "SUAREZ",
        "CASE",
        "PETTY",
        "GOULD",
        "MCFARLAND",
        "SAMPSON",
        "CARVER",
        "BRAY",
        "ROSARIO",
        "MACDONALD",
        "STOUT",
        "HESTER",
        "MELENDEZ",
        "DILLON",
        "FARLEY",
        "HOPPER",
        "GALLOWAY",
        "POTTS",
        "BERNARD",
        "JOYNER",
        "STEIN",
        "AGUIRRE",
        "OSBORN",
        "MERCER",
        "BENDER",
        "FRANCO",
        "ROWLAND",
        "SYKES",
        "BENJAMIN",
        "TRAVIS",
        "PICKETT",
        "CRANE",
        "SEARS",
        "MAYO",
        "DUNLAP",
        "HAYDEN",
        "WILDER",
        "MCKAY",
        "COFFEY",
        "MCCARTY",
        "EWING",
        "COOLEY",
        "VAUGHAN",
        "BONNER",
        "COTTON",
        "HOLDER",
        "STARK",
        "FERRELL",
        "CANTRELL",
        "FULTON",
        "LYNN",
        "LOTT",
        "CALDERON",
        "ROSA",
        "POLLARD",
        "HOOPER",
        "BURCH",
        "MULLEN",
        "FRY",
        "RIDDLE",
        "LEVY",
        "DAVID",
        "DUKE",
        "ODONNELL",
        "GUY",
        "MICHAEL",
        "BRITT",
        "FREDERICK",
        "DAUGHERTY",
        "BERGER",
        "DILLARD",
        "ALSTON",
        "JARVIS",
        "FRYE",
        "RIGGS",
        "CHANEY",
        "ODOM",
        "DUFFY",
        "FITZPATRICK",
        "VALENZUELA",
        "MERRILL",
        "MAYER",
        "ALFORD",
        "MCPHERSON",
        "ACEVEDO",
        "DONOVAN",
        "BARRERA",
        "ALBERT",
        "COTE",
        "REILLY",
        "COMPTON",
        "RAYMOND",
        "MOONEY",
        "MCGOWAN",
        "CRAFT",
        "CLEVELAND",
        "CLEMONS",
        "WYNN",
        "NIELSEN",
        "BAIRD",
        "STANTON",
        "SNIDER",
        "ROSALES",
        "BRIGHT",
        "WITT",
        "STUART",
        "HAYS",
        "HOLDEN",
        "RUTLEDGE",
        "KINNEY",
        "CLEMENTS",
        "CASTANEDA",
        "SLATER",
        "HAHN",
        "EMERSON",
        "CONRAD",
        "BURKS",
        "DELANEY",
        "PATE",
        "LANCASTER",
        "SWEET",
        "JUSTICE",
        "TYSON",
        "SHARPE",
        "WHITFIELD",
        "TALLEY",
        "MACIAS",
        "IRWIN",
        "BURRIS",
        "RATLIFF",
        "MCCRAY",
        "MADDEN",
        "KAUFMAN",
        "BEACH",
        "GOFF",
        "CASH",
        "BOLTON",
        "MCFADDEN",
        "LEVINE",
        "GOOD",
        "BYERS",
        "KIRKLAND",
        "KIDD",
        "WORKMAN",
        "CARNEY",
        "DALE",
        "MCLEOD",
        "HOLCOMB",
        "ENGLAND",
        "FINCH",
        "HEAD",
        "BURT",
        "HENDRIX",
        "SOSA",
        "HANEY",
        "FRANKS",
        "SARGENT",
        "NIEVES",
        "DOWNS",
        "RASMUSSEN",
        "BIRD",
        "HEWITT",
        "LINDSAY",
        "LE",
        "FOREMAN",
        "VALENCIA",
        "ONEIL",
        "DELACRUZ",
        "VINSON",
        "DEJESUS",
        "HYDE",
        "FORBES",
        "GILLIAM",
        "GUTHRIE",
        "WOOTEN",
        "HUBER",
        "BARLOW",
        "BOYLE",
        "MCMAHON",
        "BUCKNER",
        "ROCHA",
        "PUCKETT",
        "LANGLEY",
        "KNOWLES",
        "COOKE",
        "VELAZQUEZ",
        "WHITLEY",
        "NOEL",
        "VANG",
    ]

    _all_first_names = _female_first_names + _male_first_names



"""\
Defines Anonymizer, the principle class used to anonymize DICOM objects.
"""
from dicognito.addressanonymizer import AddressAnonymizer
from dicognito.equipmentanonymizer import EquipmentAnonymizer
from dicognito.fixedvalueanonymizer import FixedValueAnonymizer
from dicognito.idanonymizer import IDAnonymizer
from dicognito.pnanonymizer import PNAnonymizer
from dicognito.datetimeanonymizer import DateTimeAnonymizer
from dicognito.uianonymizer import UIAnonymizer
from dicognito.unwantedelements import UnwantedElementsStripper
from dicognito.randomizer import Randomizer

import pydicom
import random


class Anonymizer:
    """\
    The main class responsible for anonymizing pydicom datasets.
    New instances will anonymize instances differently, so when
    anonymizing instances from the same series, study, or patient,
    reuse an Anonymizer.

    Examples
    --------
    Anonymizing a single instance:

    >>> anonymizer = Anonymizer()
    >>> with load_instance() as dataset:
    >>>     anonymizer.anonymize(dataset)
    >>>     dataset.save_as("new filename")

    Anonymizing several instances:

    >>> anonymizer = Anonymizer()
    >>> for filename in filenames:
    >>>     with load_instance(filename) as dataset:
    >>>         anonymizer.anonymize(dataset)
    >>>         dataset.save_as("new-" + filename)
    """

    def __init__(self, id_prefix="", id_suffix="", seed=None):
        """\
        Create a new Anonymizer.

        Parameters
        ----------
        id_prefix : str
            A prefix to add to all unstructured ID fields, such as Patient
            ID, Accession Number, etc.
        id_suffix : str
            A prefix to add to all unstructured ID fields, such as Patient
            ID, Accession Number, etc.
        seed
            Not intended for general use. Seeds the data randomizer in order
            to produce consistent results. Used for testing.
        """
        minimum_offset_hours = 62 * 24
        maximum_offset_hours = 730 * 24
        randomizer = Randomizer(seed)
        address_anonymizer = AddressAnonymizer(randomizer)
        self._element_handlers = [
            UnwantedElementsStripper(
                "BranchOfService",
                "Occupation",
                "MedicalRecordLocator",
                "MilitaryRank",
                "PatientInsurancePlanCodeSequence",
                "PatientReligiousPreference",
                "PatientTelecomInformation",
                "PatientTelephoneNumbers",
                "ReferencedPatientPhotoSequence",
                "ResponsibleOrganization",
            ),
            UIAnonymizer(),
            PNAnonymizer(randomizer),
            IDAnonymizer(
                randomizer,
                id_prefix,
                id_suffix,
                "AccessionNumber",
                "OtherPatientIDs",
                "FillerOrderNumberImagingServiceRequest",
                "FillerOrderNumberImagingServiceRequestRetired",
                "FillerOrderNumberProcedure",
                "PatientID",
                "PerformedProcedureStepID",
                "PlacerOrderNumberImagingServiceRequest",
                "PlacerOrderNumberImagingServiceRequestRetired",
                "PlacerOrderNumberProcedure",
                "RequestedProcedureID",
                "ScheduledProcedureStepID",
                "StudyID",
            ),
            address_anonymizer,
            EquipmentAnonymizer(address_anonymizer),
            FixedValueAnonymizer("RequestingService", ""),
            FixedValueAnonymizer("CurrentPatientLocation", ""),
            DateTimeAnonymizer(-random.randint(minimum_offset_hours, maximum_offset_hours)),
        ]

    def anonymize(self, dataset):
        """\
        Anonymize a dataset in place. Replaces all PNs, UIs, dates and times, and
        known identifiying attributes with other vlaues.

        Parameters
        ----------
        dataset : pydicom.dataset.DataSet
            A DICOM dataset to anonymize.
        """
        dataset.file_meta.walk(self._anonymize_element)
        dataset.walk(self._anonymize_element)
        self._update_deidentification_method(dataset)
        self._update_patient_identity_removed(dataset)

    def _anonymize_element(self, dataset, data_element):
        for handler in self._element_handlers:
            if handler(dataset, data_element):
                return

    def _update_deidentification_method(self, dataset):
        if "DeidentificationMethod" not in dataset:
            dataset.DeidentificationMethod = "DICOGNITO"
            return

        existing_element = dataset.data_element("DeidentificationMethod")

        if isinstance(existing_element.value, pydicom.multival.MultiValue):
            if "DICOGNITO" not in existing_element.value:
                existing_element.value.append("DICOGNITO")
        elif existing_element.value != "DICOGNITO":
            existing_element.value = [existing_element.value, "DICOGNITO"]

    def _update_patient_identity_removed(self, dataset):
        if dataset.get("BurnedInAnnotation", "YES") == "NO":
            dataset.PatientIdentityRemoved = "YES"
